@@ -50,7 +50,7 @@ static void do_toneporta(struct module_data *m,
 		note--;
 		xc->porta.target = note_to_period(note + sub->xpo +
 			instrument->map[xc->key].xpo, sub->fin,
-			HAS_QUIRK(QUIRK_LINEAR));
+			HAS_QUIRK(QUIRK_LINEAR), xc->per_adj);
 	}
 	xc->porta.dir = xc->period < xc->porta.target ? 1 : -1;
 }
@@ -361,9 +361,7 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 			break;
 		case EX_RETRIG:		/* Retrig note */
 			SET(RETRIG);
-			if (fxp != 0) {
-				xc->retrig.val = fxp;
-			}
+			xc->retrig.val = fxp;
 			xc->retrig.count = LSN(xc->retrig.val) + 1;
 			xc->retrig.type = 0;
 			break;
@@ -448,15 +446,15 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 			p->frame_time = m->time_factor * m->rrate / p->bpm;
 		}
 		break;
+
+#ifndef LIBXMP_CORE_DISABLE_IT
 	case FX_IT_BPM:		/* Set IT BPM */
 		if (MSN(fxp) == 0) {	/* T0x - Tempo slide down by x */
-			p->bpm -= LSN(fxp);
-			if (p->bpm < 0x20)
-				p->bpm = 0x20;
+			SET(TEMPO_SLIDE);
+			xc->tempo.slide = -LSN(fxp);
 		} else if (MSN(fxp) == 1) {	/* T1x - Tempo slide up by x */
-			p->bpm += LSN(fxp);
-			if ((int32) p->bpm > 0xff)
-				p->bpm = 0xff;
+			SET(TEMPO_SLIDE);
+			xc->tempo.slide = LSN(fxp);
 		} else {
 			if (fxp < XMP_MIN_BPM)
 				fxp = XMP_MIN_BPM;
@@ -470,6 +468,8 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 			f->rowdelay_set = 1;
 		}
 		break;
+#endif
+
 	case FX_GLOBALVOL:	/* Set global volume */
 		if (fxp > m->gvolbase) {
 			p->gvol = m->gvolbase;
@@ -507,7 +507,7 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		}
 		break;
 	case FX_KEYOFF:		/* Key off */
-		xc->keyoff = fxp;
+		xc->keyoff = fxp + 1;
 		break;
 	case FX_ENVPOS:		/* Set envelope position */
 		/* FIXME: Add OpenMPT quirk */
@@ -523,6 +523,8 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 			xc->pan.slide = LSN(fxp) - MSN(fxp);
 		}
 		break;
+
+#ifndef LIBXMP_CORE_DISABLE_IT
 	case FX_IT_PANSLIDE:	/* Pan slide w/ fine pan (IT) */
 		SET(PAN_SLIDE);
 		if (fxp) {
@@ -539,6 +541,8 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 			}
 		}
 		break;
+#endif
+
 	case FX_MULTI_RETRIG:	/* Multi retrig */
 		if (fxp) {
 			xc->retrig.val = fxp;
@@ -573,6 +577,8 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 			break;
 		}
 		break;
+
+#ifndef LIBXMP_CORE_DISABLE_IT
 	case FX_TRK_VOL:	/* Track volume setting */
 		if (fxp <= m->volbase) {
 			xc->mastervol = fxp;
@@ -655,14 +661,12 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 	case FX_FLT_RESN:
 		xc->filter.resonance = fxp;
 		break;
-#if 0
-	case FX_CHORUS:
-		/* FIXME */
-		m->mod.xxc[chn].cho = fxp;
+	case FX_PANBRELLO:	/* Panbrello */
+		SET(PANBRELLO);
+		SET_LFO_NOTZERO(&xc->panbrello, LSN(fxp) << 4, MSN(fxp));
 		break;
-	case FX_REVERB:
-		/* FIXME */
-		m->mod.xxc[chn].rvb = fxp;
+	case FX_PANBRELLO_WF:	/* Panbrello waveform */
+		set_lfo_waveform(&xc->panbrello, fxp & 3);
 		break;
 #endif
 
@@ -790,6 +794,10 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		if (fxp == 0)
 			RESET_PER(VOL_SLIDE);
 		break;
+	case FX_VIBRATO2:	/* Deep vibrato (2x) */
+		SET(VIBRATO);
+		SET_LFO_NOTZERO(&xc->vibrato, LSN(fxp) << 3, MSN(fxp));
+		break;
 	case FX_SPEED_CP:	/* Set speed and ... */
 		if (fxp)
 			p->speed = fxp;
@@ -798,13 +806,6 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		xc->per_flags = 0;
 		break;
 #endif
-	case FX_PANBRELLO:	/* Panbrello */
-		SET(PANBRELLO);
-		SET_LFO_NOTZERO(&xc->panbrello, LSN(fxp) << 4, MSN(fxp));
-		break;
-	case FX_PANBRELLO_WF:	/* Panbrello waveform */
-		set_lfo_waveform(&xc->panbrello, fxp & 3);
-		break;
 	default:
 #ifndef LIBXMP_CORE_PLAYER
 		extras_process_fx(ctx, xc, chn, note, fxt, fxp, fnum);
