@@ -69,6 +69,7 @@
 #define FX_PER_CANCEL	0x7f
 #endif
 
+#ifndef LIBXMP_CORE_DISABLE_IT
 /* IT effects */
 #define FX_TRK_VOL      0x80
 #define FX_TRK_VSLIDE   0x81
@@ -76,18 +77,18 @@
 #define FX_IT_INSTFUNC	0x83
 #define FX_FLT_CUTOFF	0x84
 #define FX_FLT_RESN	0x85
-#define FX_FINE2_VIBRA	0x86
 #define FX_IT_BPM	0x87
 #define FX_IT_ROWDELAY	0x88
 #define FX_IT_PANSLIDE	0x89
 #define FX_PANBRELLO	0x8a
 #define FX_PANBRELLO_WF	0x8b
-
+#endif
 
 #ifndef LIBXMP_CORE_PLAYER
 /* MED effects */
 #define FX_HOLD_DECAY	0x90
 #define FX_SETPITCH	0x91
+#define FX_VIBRATO2	0x92
 
 /* PTM effects */
 #define FX_NSLIDE_DN	0x9c	/* IMF/PTM note slide down */
@@ -101,7 +102,7 @@
 #define FX_F_VSLIDE	0xa5	/* IMF/MDL */
 #define FX_CHORUS	0xa9	/* IMF */
 #define FX_REVERB	0xaa	/* IMF */
-#define FX_MED_DECAY	0xb1	/* MMD hold/decay */
+#define FX_MED_HOLD	0xb1	/* MMD hold/decay */
 #define FX_MEGAARP	0xb2	/* Smaksak effect 7: MegaArp */
 #endif
 
