@@ -24,9 +24,11 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#ifndef LIBXMP_CORE_PLAYER
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#endif
 #include <errno.h>
 #ifdef __native_client__
 #include <sys/syslimits.h>
@@ -40,7 +42,7 @@
 
 #ifndef LIBXMP_CORE_PLAYER
 #if !defined(HAVE_POPEN) && defined(WIN32)
-#include "../win32/ptpopen.h"
+#include "win32/ptpopen.h"
 #endif
 #include "md5.h"
 #include "extras.h"
@@ -685,13 +687,14 @@ int xmp_load_module_from_memory(xmp_context opaque, void *mem, long size)
 	if ((h = hio_open_mem(mem, size)) == NULL)
 		return -XMP_ERROR_SYSTEM;
 
-	m->filename = NULL;
-	m->basename = NULL;
-	m->size = 0;
-
 	if (ctx->state > XMP_STATE_UNLOADED)
 		xmp_release_module(opaque);
 
+	m->filename = NULL;
+	m->basename = NULL;
+	m->dirname = NULL;
+	m->size = 0;
+
 	return load_module(opaque, h, NULL);
 }
 
