@@ -1,9 +1,23 @@
-/* Extended Module Player
+/* Extended Module Player format loaders
  * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
  *
- * This file is part of the Extended Module Player and is distributed
- * under the terms of the GNU Lesser General Public License. See COPYING.LIB
- * for more information.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+ * THE SOFTWARE.
  */
 
 #include <ctype.h>
@@ -147,21 +161,6 @@ char *copy_adjust(char *s, uint8 *r, int n)
 	return s;
 }
 
-int test_name(uint8 *s, int n)
-{
-	int i;
-
-	/* ACS_Team2.mod has a backspace in instrument name */
-	for (i = 0; i < n; i++) {
-		if (s[i] > 0x7f)
-			return -1;
-		if (s[i] > 0 && s[i] < 32 && s[i] != 0x08)
-			return -1;
-	}
-
-	return 0;
-}
-
 void read_title(HIO_HANDLE *f, char *t, int s)
 {
 	uint8 buf[XMP_NAME_SIZE];
@@ -179,6 +178,23 @@ void read_title(HIO_HANDLE *f, char *t, int s)
 	copy_adjust(t, buf, s);
 }
 
+#ifndef LIBXMP_CORE_PLAYER
+
+int test_name(uint8 *s, int n)
+{
+	int i;
+
+	/* ACS_Team2.mod has a backspace in instrument name */
+	for (i = 0; i < n; i++) {
+		if (s[i] > 0x7f)
+			return -1;
+		if (s[i] > 0 && s[i] < 32 && s[i] != 0x08)
+			return -1;
+	}
+
+	return 0;
+}
+
 /*
  * Honor Noisetracker effects:
  *
@@ -218,6 +234,7 @@ void decode_noisetracker_event(struct xmp_event *event, uint8 *mod_event)
 
 	disable_continue_fx(event);
 }
+#endif
 
 void decode_protracker_event(struct xmp_event *event, uint8 *mod_event)
 {
@@ -251,6 +268,7 @@ void disable_continue_fx(struct xmp_event *event)
 	}
 }
 
+#ifndef LIBXMP_CORE_PLAYER
 #ifndef WIN32
 
 /* Given a directory, see if file exists there, ignoring case */
@@ -301,6 +319,7 @@ void get_instrument_path(struct module_data *m, char *path, int size)
 		strncpy(path, ".", size);
 	}
 }
+#endif /* LIBXMP_CORE_PLAYER */
 
 void set_type(struct module_data *m, char *fmt, ...)
 {
