@@ -60,6 +60,8 @@ static int mmd3_test(HIO_HANDLE *f, char *t, const int start)
 	return 0;
 }
 
+/* Number of octaves in IFFOCT samples */
+static const int num_oct[6] = { 5, 3, 2, 4, 6, 7 };
 
 static int mmd3_load(struct module_data *m, HIO_HANDLE *f, const int start)
 {
@@ -323,14 +325,17 @@ static int mmd3_load(struct module_data *m, HIO_HANDLE *f, const int start)
 				event = &EVENT(i, k, j);
 				event->note = e[0] & 0x7f;
 				if (event->note)
-					event->note += 24 + song.playtransp;
+					event->note += song.playtransp - 12;
+
+				if (event->note < 0 || event->note >=
+								XMP_MAX_KEYS)
+					event->note = 0;
+
 				event->ins = e[1] & 0x3f;
 
 				/* Decay */
 				if (event->ins && !event->note) {
-					event->f2t = FX_MED_DECAY;
-					event->f2p = event->ins;
-					event->ins = 0;
+					event->f2t = FX_MED_HOLD;
 				}
 
 				event->fxt = e[2];
@@ -356,8 +361,6 @@ static int mmd3_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	for (smp_idx = i = 0; i < mod->ins; i++) {
 		int smpl_offset;
 		char name[40] = "";
-		struct xmp_subinstrument *sub;
-		struct xmp_sample *xxs;
 
 		hio_seek(f, start + smplarr_offset + i * 4, SEEK_SET);
 		smpl_offset = hio_read32b(f);
@@ -398,12 +401,7 @@ static int mmd3_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		hio_seek(f, pos, SEEK_SET);
 
 		if (instr.type == -2) {			/* Hybrid */
-			int ret;
-
-			if (subinstrument_alloc(mod, i, 1) < 0)
-				return -1;
-
-			ret = mmd_load_hybrid_instrument(f, m, i,
+			int ret = mmd_load_hybrid_instrument(f, m, i,
 				smp_idx, &synth, &exp_smp, &song.sample[i]);
 
 			if (ret < 0)
@@ -435,59 +433,43 @@ static int mmd3_load(struct module_data *m, HIO_HANDLE *f, const int start)
 			continue;
 		}
 
+		if (instr.type >= 1 && instr.type <= 6) {	/* IFFOCT */
+			int ret;
+			const int oct = num_oct[instr.type - 1];
+
+			hio_seek(f, start + smpl_offset + 6, SEEK_SET);
+
+			ret = mmd_load_iffoct_instrument(f, m, i, smp_idx,
+				&instr, oct, &exp_smp, &song.sample[i]);
+
+			if (ret < 0)
+				return -1;
+
+			smp_idx += oct;
+
+			continue;
+		}
+
+		/* Filter out stereo samples */
 		if ((instr.type & ~(S_16 | STEREO)) != 0)
 			continue;
 
-		/* instr type is sample */
-		mod->xxi[i].nsm = 1;
-		if (subinstrument_alloc(mod, i, 1) < 0)
-			return -1;
+		if (instr.type == 0) {			/* Sample */
+			int ret;
 
-		sub = &mod->xxi[i].sub[0];
+			hio_seek(f, start + smpl_offset + 6, SEEK_SET);
 
-		sub->vol = song.sample[i].svol;
-		sub->pan = 0x80;
-		sub->xpo = song.sample[i].strans;
-		sub->sid = smp_idx;
-		sub->fin = exp_smp.finetune << 4;
+			ret = mmd_load_sampled_instrument(f, m, i, smp_idx,
+				&instr, &expdata, &exp_smp, &song.sample[i],
+				ver);
 
-		xxs = &mod->xxs[smp_idx];
+			if (ret < 0)
+				return -1;
 
-		xxs->len = instr.length;
-		xxs->lps = 2 * song.sample[i].rep;
-		xxs->lpe = xxs->lps + 2 * song.sample[i].replen;
-		xxs->flg = 0;
+			smp_idx++;
 
-		if (song.sample[i].replen > 1) {
-			xxs->flg |= XMP_SAMPLE_LOOP;
+			continue;
 		}
-
-		if (instr.type & S_16) {
-			xxs->flg |= XMP_SAMPLE_16BIT;
-			xxs->len >>= 1;
-			xxs->lps >>= 1;
-			xxs->lpe >>= 1;
-		}
-
-		/* STEREO means that this is a stereo sample. The sample
-		 * is not interleaved. The left channel comes first,
-		 * followed by the right channel. Important: Length
-		 * specifies the size of one channel only! The actual memory
-		 * usage for both samples is length * 2 bytes.
-		 */
-
-		D_(D_INFO "  %05x%c%05x %05x %02x %+3d %+1d ",
-				xxs->len,
-				xxs->flg & XMP_SAMPLE_16BIT ? '+' : ' ',
-				xxs->lps, xxs->lpe,
-				sub->vol, sub->xpo, sub->fin >> 4);
-
-		hio_seek(f, start + smpl_offset + 6, SEEK_SET);
-		if (load_sample(m, f, SAMPLE_FLAG_BIGEND, xxs, NULL) < 0) {
-			return -1;
-		}
-
-		smp_idx++;
 	}
 
 	hio_seek(f, start + trackvols_offset, SEEK_SET);
@@ -505,5 +487,7 @@ static int mmd3_load(struct module_data *m, HIO_HANDLE *f, const int start)
 			mod->xxc[i].pan = 0x80;
 	}
 
+	m->read_event_type = READ_EVENT_MED;
+
 	return 0;
 }
