@@ -24,6 +24,7 @@
 #include "common.h"
 #include "player.h"
 #include "virtual.h"
+#include "effects.h"
 #include "med_extras.h"
 
 #ifdef __SUNPRO_C
@@ -119,8 +120,10 @@ void med_play_extras(struct context_data *ctx, struct channel_data *xc, int chn,
 		     int new_note)
 {
 	struct module_data *m = &ctx->m;
+	struct player_data *p = &ctx->p;
 	struct med_module_extras *me;
 	struct med_channel_extras *ce;
+	struct med_instrument_extras *ie;
 	int b, jws = 0, jvs = 0, loop;
 	int temp;
 
@@ -129,9 +132,40 @@ void med_play_extras(struct context_data *ctx, struct channel_data *xc, int chn,
 
 	me = (struct med_module_extras *)m->extra;
 	ce = (struct med_channel_extras *)xc->extra;
+	ie = MED_INSTRUMENT_EXTRAS(m->mod.xxi[xc->ins]);
 
-	if (me->vol_table[xc->ins] == NULL || me->wav_table[xc->ins] == NULL)
+	/* Handle hold/decay */
+
+	/* on the first row of a held note, continue note if ce->hold is 2
+	 * (this is set after pre-fetching the next row and see if we
+	 * continue to hold. On remaining rows with hold on, we have the
+	 * FX_MED_HOLD effect and ce->hold set to 1. On the last row, see
+	 * if ce->hold_count is set (meaning that a note was held) and
+	 * ce->hold is 0 (meaning that it's not held anymore). Then
+	 * procceed with normal frame counting until decay.
+	 */
+
+	if (ce->hold_count) {		/* was held in the past */
+		if (!ce->hold && p->frame >= ie->hold) { /* but not now */
+			SET_NOTE(NOTE_FADEOUT);
+			ce->hold_count = 0;
+		}
+	} else if (ie->hold) {		/* has instrument hold */
+		if (p->frame >= ie->hold && ce->hold == 0) {
+			SET_NOTE(NOTE_FADEOUT);
+		}
+	}
+
+	if (p->frame == (p->speed - 1) && ce->hold != 2) {
+		ce->hold = 0;
+	}
+
+	/* Handle synth */
+
+	if (me->vol_table[xc->ins] == NULL || me->wav_table[xc->ins] == NULL) {
+		ce->volume = 64;  /* we need this in extras_get_volume() */
 		return;
+	}
 
 	if (new_note) {
 		ce->arp = ce->aidx = 0;
@@ -139,8 +173,8 @@ void med_play_extras(struct context_data *ctx, struct channel_data *xc, int chn,
 		ce->vp = ce->vc = ce->vw = 0;
 		ce->wp = ce->wc = ce->ww = 0;
 		ce->vv = 0;
-		ce->vs = MED_INSTRUMENT_EXTRAS(m->mod.xxi[xc->ins])->vts;
-		ce->ws = MED_INSTRUMENT_EXTRAS(m->mod.xxi[xc->ins])->wts;
+		ce->vs = ie->vts;
+		ce->ws = ie->wts;
 	}
 
 	if (ce->vs > 0 && ce->vc-- == 0) {
@@ -353,3 +387,30 @@ void med_release_module_extras(struct module_data *m)
 	free(m->extra);
 }
 
+void med_extras_process_fx(struct context_data *ctx, struct channel_data *xc,
+			int chn, uint8 note, uint8 fxt, uint8 fxp, int fnum)
+{
+	switch (fxt) {
+        case FX_MED_HOLD:
+		MED_CHANNEL_EXTRAS((*xc))->hold_count++;
+		MED_CHANNEL_EXTRAS((*xc))->hold = 1;
+		break;
+	}
+}
+
+void med_hold_hack(struct context_data *ctx, int pat, int chn, int row)
+{
+	struct module_data *m = &ctx->m;
+	struct xmp_module *mod = &m->mod;
+	const int num_rows = mod->xxt[TRACK_NUM(pat, chn)]->rows;
+
+	if (row + 1 < num_rows) {
+		struct player_data *p = &ctx->p;
+		struct xmp_event *event = &EVENT(pat, chn, row + 1);
+		struct channel_data *xc = &p->xc_data[chn];
+
+		if (event->f2t == FX_MED_HOLD) {
+			MED_CHANNEL_EXTRAS(*xc)->hold = 2;
+		}
+	}
+}
