@@ -180,13 +180,14 @@ static int check_delay(struct context_data *ctx, struct xmp_event *e, int chn)
     do_delay:
 	memcpy(&xc->delayed_event, e, sizeof (struct xmp_event));
 
-	if (e->ins)
+	if (e->ins) {
 		xc->delayed_ins = e->ins;
+	}
 
 	if (HAS_QUIRK(QUIRK_RTDELAY)) {
 		if (e->note == 0)
 			xc->delayed_event.note = xc->key + 1;
-		if (e->ins == 0)
+		if (e->ins == 0 && !HAS_QUIRK(QUIRK_MLKDLY))
 			xc->delayed_event.ins = xc->old_ins;
 	}
 
@@ -209,7 +210,8 @@ static inline void read_row(struct context_data *ctx, int pat, int row)
 	}
 
 	for (chn = 0; chn < mod->chn; chn++) {
-		if (row < mod->xxt[TRACK_NUM(pat, chn)]->rows) {
+		const int num_rows = mod->xxt[TRACK_NUM(pat, chn)]->rows;
+		if (row < num_rows) {
 			event = &EVENT(pat, chn, row);
 		} else {
 			event = (struct xmp_event *)&empty_event;
@@ -218,6 +220,9 @@ static inline void read_row(struct context_data *ctx, int pat, int row)
 		if (check_delay(ctx, event, chn) == 0) {
 			if (!f->rowdelay_set || f->rowdelay > 0) {
 				read_event(ctx, event, chn);
+#ifndef LIBXMP_CORE_PLAYER
+				med_hold_hack(ctx, pat, chn, row);
+#endif
 			}
 		}
 	}
@@ -394,7 +399,9 @@ static void process_volume(struct context_data *ctx, int chn, int t, int act)
 
 static void process_frequency(struct context_data *ctx, int chn, int t, int act)
 {
+#ifndef LIBXMP_CORE_DISABLE_IT
 	struct mixer_data *s = &ctx->s;
+#endif
 	struct player_data *p = &ctx->p;
 	struct module_data *m = &ctx->m;
 	struct channel_data *xc = &p->xc_data[chn];
@@ -402,7 +409,10 @@ static void process_frequency(struct context_data *ctx, int chn, int t, int act)
 	double period;
 	int linear_bend;
 	int frq_envelope;
-	int arp, vibrato, cutoff, resonance;
+	int arp, vibrato;
+#ifndef LIBXMP_CORE_DISABLE_IT
+	int cutoff, resonance;
+#endif
 	int end;
 
 	instrument = get_instrument(ctx, xc->ins);
@@ -410,17 +420,19 @@ static void process_frequency(struct context_data *ctx, int chn, int t, int act)
 	frq_envelope = get_envelope(&instrument->fei, xc->f_idx, 0, &end);
 	xc->f_idx = update_envelope(&instrument->fei, xc->f_idx, DOENV_RELEASE);
 
+#ifndef LIBXMP_CORE_PLAYER
 	/* Do note slide */
 
 	if (TEST(NOTE_SLIDE)) {
 		if (xc->noteslide.count == 0) {
 			xc->note += xc->noteslide.slide;
 			xc->period = note_to_period(xc->note, xc->finetune,
-						    HAS_QUIRK(QUIRK_LINEAR));
+					HAS_QUIRK(QUIRK_LINEAR), xc->per_adj);
 			xc->noteslide.count = xc->noteslide.speed;
 		}
 		xc->noteslide.count--;
 	}
+#endif
 
 	/* Instrument vibrato */
 	vibrato = get_lfo(&xc->insvib.lfo, (1024 * (1 + xc->insvib.sweep)));
@@ -454,7 +466,7 @@ static void process_frequency(struct context_data *ctx, int chn, int t, int act)
 
 	linear_bend = period_to_bend(period + vibrato, xc->note,
 				HAS_QUIRK(QUIRK_MODRNG), xc->gliss,
-				HAS_QUIRK(QUIRK_LINEAR));
+				HAS_QUIRK(QUIRK_LINEAR), xc->per_adj);
 
 	/* Envelope */
 
@@ -482,6 +494,8 @@ static void process_frequency(struct context_data *ctx, int chn, int t, int act)
 
 	virt_setbend(ctx, chn, linear_bend);
 
+#ifndef LIBXMP_CORE_DISABLE_IT
+
 	/* Process filter */
 
 	if (!HAS_QUIRK(QUIRK_FILTER)) {
@@ -508,6 +522,8 @@ static void process_frequency(struct context_data *ctx, int chn, int t, int act)
 
 	/* Always set cutoff */
 	virt_seteffect(ctx, chn, DSP_EFFECT_CUTOFF, cutoff);
+
+#endif
 }
 
 static void process_pan(struct context_data *ctx, int chn, int t, int act)
@@ -648,7 +664,7 @@ static void update_frequency(struct context_data *ctx, int chn, int t)
 		if (TEST(FINE_NSLIDE)) {
 			xc->note += xc->noteslide.fslide;
 			xc->period = note_to_period(xc->note, xc->finetune,
-						    HAS_QUIRK(QUIRK_LINEAR));
+					 HAS_QUIRK(QUIRK_LINEAR), xc->per_adj);
 		}
 #endif
 	}
@@ -702,6 +718,12 @@ static void play_channel(struct context_data *ctx, int chn, int t)
 
 	xc->info_finalvol = 0;
 
+	/* IT tempo slide */
+	if (TEST(TEMPO_SLIDE) && p->frame) {
+		p->bpm += xc->tempo.slide;
+		CLAMP(p->bpm, 0x20, 0xff);
+	}
+
 	/* Do delay */
 	if (xc->delay > 0) {
 		if (--xc->delay == 0) {
@@ -732,7 +754,11 @@ static void play_channel(struct context_data *ctx, int chn, int t)
 
 	/* Do cut/retrig */
 	if (TEST(RETRIG)) {
-		if (--xc->retrig.count <= 0) {
+		int cond = HAS_QUIRK(QUIRK_S3MRTG) ?
+				--xc->retrig.count <= 0 :
+				--xc->retrig.count == 0;
+
+		if (cond) {
 			if (xc->retrig.type < 0x10) {
 				/* don't retrig on cut */
 				virt_voicepos(ctx, chn, 0);
@@ -754,7 +780,7 @@ static void play_channel(struct context_data *ctx, int chn, int t)
 
 	/* Do keyoff */
 	if (xc->keyoff) {
-		if (!--xc->keyoff)
+		if (--xc->keyoff == 0)
 			SET_NOTE(NOTE_RELEASE);
 	}
 
@@ -833,8 +859,9 @@ static void next_order(struct context_data *ctx)
 	/* Reset persistent effects at new pattern */
 	if (HAS_QUIRK(QUIRK_PERPAT)) {
 		int chn;
-		for (chn = 0; chn < mod->chn; chn++)
+		for (chn = 0; chn < mod->chn; chn++) {
 			p->xc_data[chn].per_flags = 0;
+		}
 	}
 #endif
 }
@@ -885,6 +912,8 @@ static void next_row(struct context_data *ctx)
 	}
 }
 
+#ifndef LIBXMP_CORE_DISABLE_IT
+
 /*
  * Set note action for virt_pastnote
  */
@@ -904,6 +933,8 @@ void player_set_fadeout(struct context_data *ctx, int chn)
 	SET_NOTE(NOTE_FADEOUT);
 }
 
+#endif
+
 
 int xmp_start_player(xmp_context opaque, int rate, int format)
 {
