@@ -50,6 +50,7 @@ struct retrig_control {
 #define RETRIG		(1 << 18)
 #define PANBRELLO	(1 << 19)
 #define GVOL_SLIDE	(1 << 20)
+#define TEMPO_SLIDE	(1 << 21)
 
 #define NOTE_FADEOUT	(1 << 0)
 #define NOTE_RELEASE	(1 << 1)
@@ -70,6 +71,7 @@ struct channel_data {
 	int note;		/* Note number */
 	int key;		/* Key number */
 	double period;		/* Amiga or linear period */
+	double per_adj;		/* MED period/pitch adjustment factor hack */
 	int finetune;		/* Guess what */
 	int ins;		/* Instrument number */
 	int old_ins;		/* Last instruemnt */
@@ -157,12 +159,18 @@ struct channel_data {
 		int memory;	/* Pan slide effect memory */
 	} pan;	
 
+#ifndef LIBXMP_CORE_PLAYER
 	struct {
 		int slide;	/* PTM note slide amount */
 		int fslide;	/* OKT fine note slide amount */
 		int speed;	/* PTM note slide speed */
 		int count;	/* PTM note slide counter */
 	} noteslide;
+#endif
+
+	struct {
+		int slide;	/* IT tempo slide */
+	} tempo;
 
 	struct {
 		int speed;
@@ -175,7 +183,9 @@ struct channel_data {
 		int resonance;	/* IT filter resonance */
 	} filter;
 
+#ifndef LIBXMP_CORE_PLAYER
 	void *extra;
+#endif
 
 	struct xmp_event delayed_event;
 	int delayed_ins;	/* IT save instrument emulation */
