@@ -109,9 +109,11 @@ static void set_effect_defaults(struct context_data *ctx, int note,
 		set_lfo_phase(&xc->tremolo, 0);
 
 		xc->porta.target = note_to_period(note,
-				xc->finetune, HAS_QUIRK(QUIRK_LINEAR));
-		if (xc->period < 1 || !is_toneporta)
+				xc->finetune, HAS_QUIRK(QUIRK_LINEAR),
+				xc->per_adj);
+		if (xc->period < 1 || !is_toneporta) {
 			xc->period = xc->porta.target;
+		}
 	}
 
 	xc->delay = 0;
@@ -164,7 +166,7 @@ static int read_event_mod(struct context_data *ctx, struct xmp_event *e, int chn
 		xc->fadeout = 0x10000;	/* for painlace.mod pat 0 ch 3 echo */
 		xc->per_flags = 0;
 		xc->offset_val = 0;
-		RESET_NOTE(NOTE_RELEASE);
+		RESET_NOTE(NOTE_RELEASE|NOTE_FADEOUT);
 
 		if (IS_VALID_INSTRUMENT(ins)) {
 			if (is_toneporta) {
@@ -181,13 +183,6 @@ static int read_event_mod(struct context_data *ctx, struct xmp_event *e, int chn
 			new_invalid_ins = 1;
 			virt_resetchannel(ctx, chn);
 		}
-
-#ifndef LIBXMP_CORE_PLAYER
-		if (HAS_MED_CHANNEL_EXTRAS(*xc)) {
-			MED_CHANNEL_EXTRAS(*xc)->arp = 0;
-			MED_CHANNEL_EXTRAS(*xc)->aidx = 0;
-		}
-#endif
 	}
 
 	/* Check note */
@@ -306,7 +301,7 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 		use_ins_vol = 1;
 		xc->fadeout = 0x10000;
 		xc->per_flags = 0;
-		RESET_NOTE(NOTE_RELEASE);
+		RESET_NOTE(NOTE_RELEASE|NOTE_FADEOUT);
 
 		if (IS_VALID_INSTRUMENT(ins - 1)) {
 			if (!is_toneporta)
@@ -387,7 +382,7 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 		xc->fadeout = 0x10000;
 		RESET_NOTE(NOTE_END);
 		if (~mod->xxi[xc->ins].aei.flg & XMP_ENVELOPE_ON) {
-			RESET_NOTE(NOTE_RELEASE);
+			RESET_NOTE(NOTE_RELEASE|NOTE_FADEOUT);
 		}
 
 		sub = get_subinstrument(ctx, xc->ins, key);
@@ -497,7 +492,7 @@ static int read_event_st3(struct context_data *ctx, struct xmp_event *e, int chn
 		xc->fadeout = 0x10000;
 		xc->per_flags = 0;
 		xc->offset_val = 0;
-		RESET_NOTE(NOTE_RELEASE);
+		RESET_NOTE(NOTE_RELEASE|NOTE_FADEOUT);
 
 		if (IS_VALID_INSTRUMENT(ins)) {
 			/* valid ins */
@@ -610,6 +605,8 @@ static int read_event_st3(struct context_data *ctx, struct xmp_event *e, int chn
 	return 0;
 }
 
+#ifndef LIBXMP_CORE_DISABLE_IT
+
 static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 {
 	struct player_data *p = &ctx->p;
@@ -726,7 +723,7 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 			 */
 			if (not_same_ins || TEST_NOTE(NOTE_END)) {
 				SET(NEW_INS);
-				RESET_NOTE(NOTE_RELEASE);
+				RESET_NOTE(NOTE_RELEASE|NOTE_FADEOUT);
 			} else {
 				key = 0;
 			}
@@ -825,6 +822,156 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 	return 0;
 }
 
+#endif
+
+#ifndef LIBXMP_CORE_PLAYER
+
+static int read_event_med(struct context_data *ctx, struct xmp_event *e, int chn)
+{
+	struct player_data *p = &ctx->p;
+	struct module_data *m = &ctx->m;
+	struct xmp_module *mod = &m->mod;
+	struct channel_data *xc = &p->xc_data[chn];
+	int note;
+	struct xmp_subinstrument *sub;
+	int new_invalid_ins = 0;
+	int is_toneporta;
+	int use_ins_vol;
+
+	xc->flags = 0;
+	note = -1;
+	is_toneporta = 0;
+	use_ins_vol = 0;
+
+	if (e->fxt == FX_TONEPORTA || e->fxt == FX_TONE_VSLIDE) {
+		is_toneporta = 1;
+	}
+
+	/* Check instrument */
+
+	if (e->ins && e->note) {
+		int ins = e->ins - 1;
+		use_ins_vol = 1;
+		SET(NEW_INS);
+		xc->fadeout = 0x10000;
+		xc->offset_val = 0;
+		RESET_NOTE(NOTE_RELEASE|NOTE_FADEOUT);
+
+		if (IS_VALID_INSTRUMENT(ins)) {
+			if (is_toneporta) {
+				/* Get new instrument volume */
+				sub = get_subinstrument(ctx, ins, e->note);
+				if (sub != NULL) {
+					xc->volume = sub->vol;
+					use_ins_vol = 0;
+				}
+			} else {
+				xc->ins = ins;
+			}
+		} else {
+			new_invalid_ins = 1;
+			virt_resetchannel(ctx, chn);
+		}
+
+		MED_CHANNEL_EXTRAS(*xc)->arp = 0;
+		MED_CHANNEL_EXTRAS(*xc)->aidx = 0;
+	} else {
+		/* Hold */
+		if (e->ins && !e->note) {
+			use_ins_vol = 1;
+		}
+	}
+
+	/* Check note */
+
+	if (e->note) {
+		SET(NEW_NOTE);
+
+		if (e->note == XMP_KEY_OFF) {
+			SET_NOTE(NOTE_RELEASE);
+			use_ins_vol = 0;
+		} else if (!is_toneporta && IS_VALID_INSTRUMENT(xc->ins)) {
+			struct xmp_instrument *xxi = &mod->xxi[xc->ins];
+
+			xc->key = e->note - 1;
+			RESET_NOTE(NOTE_END);
+		
+			xc->per_adj = 0.0;
+			if (xxi->nsm > 1 && HAS_MED_INSTRUMENT_EXTRAS(*xxi)) {
+				/* synth or iffoct */
+				if (MED_INSTRUMENT_EXTRAS(*xxi)->vts == 0 &&
+				    MED_INSTRUMENT_EXTRAS(*xxi)->wts == 0) {
+					/* iffoct */
+					xc->per_adj = 2.0;
+				}
+			}
+	
+			sub = get_subinstrument(ctx, xc->ins, xc->key);
+	
+			if (!new_invalid_ins && sub != NULL) {
+				int transp = xxi->map[xc->key].xpo;
+				int smp;
+	
+				note = xc->key + sub->xpo + transp;
+				smp = sub->sid;
+	
+				if (mod->xxs[smp].len == 0) {
+					smp = -1;
+				}
+	
+				if (smp >= 0 && smp < mod->smp) {
+					set_patch(ctx, chn, xc->ins, smp, note);
+					xc->smp = smp;
+				}
+			} else {
+				xc->flags = 0;
+				use_ins_vol = 0;
+			}
+		}
+	}
+
+	sub = get_subinstrument(ctx, xc->ins, xc->key);
+
+	set_effect_defaults(ctx, note, sub, xc, is_toneporta);
+	if (e->ins && sub != NULL)
+		reset_envelopes(ctx, xc);
+
+	/* Process new volume */
+	if (e->vol) {
+		xc->volume = e->vol - 1;
+		SET(NEW_VOL);
+	}
+
+	/* Secondary effect handled first */
+	process_fx(ctx, xc, chn, e->note, e->f2t, e->f2p, 1);
+	process_fx(ctx, xc, chn, e->note, e->fxt, e->fxp, 0);
+
+	if (TEST(NEW_VOL))
+		use_ins_vol = 0;
+
+	if (sub == NULL) {
+		return 0;
+	}
+
+	if (note >= 0) {
+		xc->note = note;
+
+		virt_voicepos(ctx, chn, xc->offset_val);
+		if (TEST(OFFSET) && p->flags & XMP_FLAGS_FX9BUG)
+			xc->offset_val <<= 1;
+		RESET(OFFSET);
+	}
+
+	if (use_ins_vol) {
+		xc->volume = sub->vol;
+		SET(NEW_VOL);
+	}
+
+	return 0;
+}
+
+#endif
+
 static int read_event_smix(struct context_data *ctx, struct xmp_event *e, int chn)
 {
 	struct player_data *p = &ctx->p;
@@ -875,7 +1022,7 @@ static int read_event_smix(struct context_data *ctx, struct xmp_event *e, int ch
 			smp = -1;
 		if (smp >= 0 && smp < smix->smp) {
 			smp += mod->smp;
-			virt_setpatch(ctx, chn, xc->ins, smp, note, 0, 0, 0);
+			set_patch(ctx, chn, xc->ins, smp, note);
 			xc->smp = smp;
 		}
 	} else {
@@ -922,8 +1069,14 @@ int read_event(struct context_data *ctx, struct xmp_event *e, int chn)
 		return read_event_ft2(ctx, e, chn);
 	case READ_EVENT_ST3:
 		return read_event_st3(ctx, e, chn);
+#ifndef LIBXMP_CORE_DISABLE_IT
 	case READ_EVENT_IT:
 		return read_event_it(ctx, e, chn);
+#endif
+#ifndef LIBXMP_CORE_PLAYER
+	case READ_EVENT_MED:
+		return read_event_med(ctx, e, chn);
+#endif
 	default:
 		return read_event_mod(ctx, e, chn);
 	}
